(*====================================================================
    File ibds.dpr

    InterBase Data Supplier.
    Project file.

    Copyright  2004-2008, PSTG
    http://www.polesoft.ru
    mailto:support@polesoft.ru

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
======================================================================
$Id: ibds.dpr,v 1.9 2009/01/26 17:15:41 ariafan Exp $
======================================================================*)
program ibds;
{$APPTYPE CONSOLE}

uses
  SysUtils, DB, rcrt,
  SqlConst, IniFiles, windows,
  dm in 'dm.pas' {DM1: TDataModule};
var
  ini: TIniFile;
  f: text;
  i,cr,sr: integer;
  s, comm, sec: string;
  rk: boolean;

resourcestring
  rsUsage =
    'Usage:  ibds <action> <dsname>[(param,..)] [[[[<dbname>] <username>] <password>] <rolename>]'+ sLineBreak +
    '        <action>    s - select records into text file,'+ sLineBreak +
    '                    i - insert records into table'+ sLineBreak +
    '                    e - execute SQL command'+ sLineBreak +
    '        <dsname>    dataset(table) name'+ sLineBreak +
    '        <dbname>    full database filename'+ sLineBreak +
    '        <username>  user name'+ sLineBreak +
    '        <password>  user password'+ sLineBreak +
    '        <rolename>  role name'+ sLineBreak +
    'Note: uppercased <action> don''t paused after executing.'+ sLineBreak +
    'Examples:  ibds s table1'+ sLineBreak +
    '           ibds i table2 myib:/db/mydb.gdb iamuser dontknow'+ sLineBreak +
    '           ibds E myproc(parval1,parval2) myib:/db/mydb.gdb iamuser dontknow';
  rsSelectCmd = 'SELECT command:';
  rsInsertCmd = '"INSERT INTO %s" command.';
  rsSQLCmd = 'SQL command:';
  rsConnect = 'Connecting to %s...';
  rsPrepare = 'Preparing...';
  rsSelect = 'Selecting...';
  rsFetch = 'Fetching...';
  rsInsert = 'Inserting...';
  rsExecute = 'Executing...';
  rsSkipped = '%d record(s) skipped.';
  rsFetched = '%d record(s) fetched.';
  rsInserted = '%d record(s) inserted.';
  rsTotal = 'Total %s';
  rsUnkAction = 'unknown action %s.';
  rsError = 'Error: %s';
  rsAnyKey = 'Press any key...';
  rsOK = 'OK';

procedure AssignParams(Params: TParams; paramstr: string);
var
  i: integer;
  ss: string;
begin
  if pos('(', paramstr) > 0 then begin
    paramstr := copy(paramstr, pos('(', paramstr) + 1, Length(paramstr));
    paramstr := copy(paramstr, 1, pos(')', paramstr) - 1);
  end else
    paramstr := '';
  for i := 0 to Params.Count - 1 do begin
    if pos(',', paramstr) > 0 then
      ss := Copy(paramstr, 1, Pos(',', paramstr) - 1)
    else
      ss := paramstr;
    with Params[i] do begin
      if ss = '' then
        Clear
      else
        AsString := ss;
      Bound := True;
      if IsNull then ss := 'NULL'     // don't localize
      else ss := AsString;
      Writeln(Format('%s=%s', [Name, ss]));
    end;
    paramstr := copy(paramstr, pos(',', paramstr) + 1, Length(paramstr));
  end;
end;

function ToOEM(s: string): string;
begin
  SetLength(Result, Length(s));
  CharToOem(PChar(s), PChar(Result));
end;

begin
// don't localize
  Writeln('Polaris Utilities 1.4, InterBase Data Supplier 1.4.4');
  Writeln('Copyright (c) 2005-2008, PSTG     http://www.polesoft.ru');
  Writeln('This program is distributed under GNU General Public License.');

  if ParamCount < 2 then begin
    Writeln(ToOEM(rsUsage));
    writeln('');
    write(ToOEM(rsAnyKey));
    ReadKey;
    Writeln('');
    halt(1);
  end;
  Writeln('');
  rk := ParamStr(1) <> UpperCase(ParamStr(1));
  ini := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'ibds.ini'); // don't localize
  try
  try
  with TDM1.Create(nil) do try
    with IBD do begin
      if ParamCount > 2 then
        DatabaseName := ParamStr(3)
      else
        DatabaseName := ini.ReadString('db', 'file', ''); // don't localize
      if ParamCount > 3 then
        Params.Values[szUSERNAME] := ParamStr(4)
      else
        Params.Values[szUSERNAME] := ini.ReadString('db', 'username', 'sysdba');  // don't localize
      if ParamCount > 4 then
        Params.Values[szPASSWORD] := ParamStr(5)
      else
        Params.Values[szPASSWORD] := ini.ReadString('db', 'password', 'masterke'); // don't localize
      if ParamCount > 5 then
        Params.Values['sql_role_name'] := ParamStr(6) // don't localize
      else
        Params.Values['sql_role_name'] := ini.ReadString('db', 'rolename', ''); // don't localize
      Params.Values['lc_ctype'] := ini.ReadString('db', 'charset', 'WIN1251'); // don't localize
      Write(ToOEM(Format(rsConnect, [DatabaseName])));
      Open;
      Writeln(ToOEM(rsOK));
    end;
    // cut params from the dataset name
    sec := ParamStr(2);
    if Pos('(', sec) > 0 then
      sec := copy(sec, 1, pos('(', sec) - 1);

    cr := StrToIntDef(ini.ReadString(sec, 'commitrecords', '10000'), 10000); // don't localize
    sr := StrToIntDef(ini.ReadString(sec, 'skiprecords', '0'), 0); // don't localize
    AssignFile(f, sec + '.txt'); // don't localize
    case UpperCase(ParamStr(1))[1] of
      'S': begin
             Rewrite(f);
             try
               with IBQ do begin
                 SQL.Text := ini.ReadString(sec, 'select', ''); // don't localize
                 if SQL.Text = '' then
                   SQL.Text := Format('select %s from %s where %s',     // don't localize
                                      [ini.ReadString(sec, 'fields', '*'), sec,
                                       ini.ReadString(sec, 'where', '0=0')]);
                 Writeln(ToOEM(rsSelectCmd));
                 Writeln(ToOEM(SQL.Text));
                 Write(ToOEM(rsPrepare));
                 Prepare;
                 if Params.Count = 0 then Writeln(ToOEM(rsOK))
                 else begin
                   Writeln('');
                   AssignParams(IBQ.Params, ParamStr(2));
                 end;
                 Write(ToOEM(rsSelect));
                 Open;
                 Writeln(ToOEM(rsOK));
                 Writeln(ToOEM(rsFetch));
                 SysUtils.DecimalSeparator := '.';
                 while not eof do begin
                   if (recno > sr) then begin
                     if (recno - 1 = sr) and (sr > 0) then
                       Writeln(ToOEM(Format(rsSkipped, [sr])));
                     s := '';
                     for i:=0 to fieldcount-1 do begin
                       if fields[i].isnull then
                         s := s+',NULL' // don't localize
                       else
                         case fields[i].DataType of
                           ftString,ftDate,ftDateTime,ftTimestamp: s := s+','#39+
                                  StringReplace(fields[i].AsString, #39, #39#39, [rfReplaceAll])+#39;
                         else
                           s := s+','+StringReplace(fields[i].AsString, #39, #39#39, [rfReplaceAll]);
                         end;
                     end;
                     writeln(f, Copy(s, 2, Length(s)));
                     if (recno > sr) and (((recno - sr) mod cr) = 0) then begin
                       flush(f);
                       Writeln(ToOEM(Format(rsFetched, [recno - sr])));
                     end;
                   end;
                   next;
                 end;
               end;
             finally
               Writeln(ToOEM(Format(rsTotal, [Format(rsFetched, [IBQ.recno - sr])])));
               CloseFile(F);
             end;
           end;
      'I': begin
             Reset(f);
             i := 0;
             try
               Writeln(Format(ToOEM(rsInsertCmd), [sec]));
               s := ini.ReadString(ParamStr(2), 'fields', '*'); // don't localize
               if s = '*' then
                 comm := Format('insert into %s values (', [sec]) // don't localize
               else
                 comm := Format('insert into %s (%s) values (', // don't localize
                                [ParamStr(2), ini.ReadString(sec, 'fields', '*')]);
               Writeln(ToOEM(rsInsert));
               IBT.StartTransaction;
               while not eof(f) do begin
                 Readln(f, s);
                 if Trim(s) = '' then Continue;
                 if (i + 1 > sr) then begin
                   if (i = sr) and (sr > 0) then
                     Writeln(ToOEM(Format(rsSkipped, [i])));
                   with IBSQL do begin
                     SQL.Text := comm + s + ')';
                     ExecQuery;
                     if (i >= sr) and (((i - sr + 1) mod cr) = 0) then begin
                       IBT.CommitRetaining;
                       Writeln(ToOEM(Format(rsInserted, [i - sr + 1])));
                     end;
                   end;
                 end;
                 Inc(i);
               end;
             finally
               IBT.Commit;
               Writeln(ToOEM(Format(rsTotal, [Format(rsInserted, [i - sr])])));
               CloseFile(F);
             end;
           end;
      'E': begin
             Writeln(ToOEM(rsSQLCmd));
             with IBQ do begin
               SQL.Text := ini.ReadString(sec, 'command', ''); // don't localize
               Writeln(ToOEM(SQL.Text));
               Write(ToOEM(rsPrepare));
               Prepare;
               if Params.Count = 0 then Writeln(ToOEM(rsOK))
               else begin
                 Writeln('');
                 AssignParams(IBQ.Params, ParamStr(2));
               end;
               Write(ToOEM(rsExecute));
               ExecSQL;
               Writeln(ToOEM(rsOK));
             end;
           end
      else
        raise Exception.CreateFmt(ToOEM(rsUnkAction), [ParamStr(1)]);
    end;
    IBD.Close;
  finally
    Free;
  end;
  finally
    ini.Free;
  end;
  except
    on E:Exception do begin
      Writeln('');
      Writeln(ToOEM(Format(rsError, [E.Message])));
      ExitCode := 2;
    end;
  end;
  if rk then begin
    Write(ToOEM(rsAnyKey));
    ReadKey;
    Writeln('');
  end;
end.
